@@ -15,15 +15,17 @@
 # specific language governing permissions and limitations
 # under the License.
 """Run ephemeral Docker Swarm services"""
-from typing import Optional
+from typing import TYPE_CHECKING, List, Optional, Union
 
-import requests
 from docker import types
 
 from airflow.exceptions import AirflowException
 from airflow.providers.docker.operators.docker import DockerOperator
 from airflow.utils.strings import get_random_string
 
+if TYPE_CHECKING:
+    from airflow.utils.context import Context
+
 
 class DockerSwarmOperator(DockerOperator):
     """
@@ -93,15 +95,47 @@ class DockerSwarmOperator(DockerOperator):
         Supported only if the Docker engine is using json-file or journald logging drivers.
         The `tty` parameter should be set to use this with Python applications.
     :type enable_logging: bool
+    :param configs: List of docker configs to be exposed to the containers of the swarm service.
+        The configs are ConfigReference objects as per the docker api
+        [https://docker-py.readthedocs.io/en/stable/services.html#docker.models.services.ServiceCollection.create]_
+    :type configs: List[docker.types.ConfigReference]
+    :param secrets: List of docker secrets to be exposed to the containers of the swarm service.
+        The secrets are SecretReference objects as per the docker create_service api.
+        [https://docker-py.readthedocs.io/en/stable/services.html#docker.models.services.ServiceCollection.create]_
+    :type secrets: List[docker.types.SecretReference]
+    :param mode: Indicate whether a service should be deployed as a replicated or global service,
+        and associated parameters
+    :type mode: docker.types.ServiceMode
+    :param networks: List of network names or IDs or NetworkAttachmentConfig to attach the service to.
+    :type networks: List[Union[str, NetworkAttachmentConfig]]
+    :param placement: Placement instructions for the scheduler. If a list is passed instead,
+        it is assumed to be a list of constraints as part of a Placement object.
+    :type placement: Union[types.Placement, List[types.Placement]]
     """
 
-    def __init__(self, *, image: str, enable_logging: bool = True, **kwargs) -> None:
+    def __init__(
+        self,
+        *,
+        image: str,
+        enable_logging: bool = True,
+        configs: Optional[List[types.ConfigReference]] = None,
+        secrets: Optional[List[types.SecretReference]] = None,
+        mode: Optional[types.ServiceMode] = None,
+        networks: Optional[List[Union[str, types.NetworkAttachmentConfig]]] = None,
+        placement: Optional[Union[types.Placement, List[types.Placement]]] = None,
+        **kwargs,
+    ) -> None:
         super().__init__(image=image, **kwargs)
 
         self.enable_logging = enable_logging
         self.service = None
+        self.configs = configs
+        self.secrets = secrets
+        self.mode = mode
+        self.networks = networks
+        self.placement = placement
 
-    def execute(self, context) -> None:
+    def execute(self, context: 'Context') -> None:
         self.cli = self._get_cli()
 
         self.environment['AIRFLOW_TMP_DIR'] = self.tmp_dir
@@ -121,12 +155,17 @@ class DockerSwarmOperator(DockerOperator):
                     env=self.environment,
                     user=self.user,
                     tty=self.tty,
+                    configs=self.configs,
+                    secrets=self.secrets,
                 ),
                 restart_policy=types.RestartPolicy(condition='none'),
                 resources=types.Resources(mem_limit=self.mem_limit),
+                networks=self.networks,
+                placement=self.placement,
             ),
             name=f'airflow-{get_random_string()}',
             labels={'name': f'airflow__{self.dag_id}__{self.task_id}'},
+            mode=self.mode,
         )
 
         self.log.info('Service started: %s', str(self.service))
@@ -173,12 +212,6 @@ class DockerSwarmOperator(DockerOperator):
         while True:
             try:
                 log = next(logs)
-            # TODO: Remove this clause once https://github.com/docker/docker-py/issues/931 is fixed
-            except requests.exceptions.ConnectionError:
-                # If the service log stream stopped sending messages, check if it the service has
-                # terminated.
-                if self._has_service_terminated():
-                    break
             except StopIteration:
                 # If the service log stream terminated, stop fetching logs further.
                 break
